"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const C = require("@collectable/core");
const internals_1 = require("../internals");
/**
 * Removes the specified key from the tree. If the key was not in the tree, no changes are made, and the original tree
 * is returned.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {K} key The key of the entry to be removed
 * @param {RedBlackTreeStructure<K, V>} tree The tree to be updated
 * @returns {RedBlackTreeStructure<K, V>} An updated copy of the tree, or the same tree if the input tree was already mutable
 */
function remove(key, tree) {
    if (tree._size === 0) {
        return tree;
    }
    var originalTree = tree;
    tree = C.modify(tree);
    C.modifyProperty(tree, '_root');
    var p = internals_1.findPath(tree, key, tree._root, tree._compare);
    if (p.next !== 0 /* NONE */) {
        internals_1.PathNode.release(p, tree._root);
        return originalTree;
    }
    if (tree._size === 1) {
        internals_1.PathNode.release(p, tree._root);
        if (tree !== originalTree) {
            return internals_1.createTree(tree._compare);
        }
        tree._size = 0;
        tree._root = internals_1.NONE;
        return tree;
    }
    var current = p.node;
    var hasRight = !internals_1.isNone(current._right);
    var hasLeft = !internals_1.isNone(current._left);
    var debt = 0;
    if (hasRight || hasLeft) {
        if (hasRight) {
            if (hasLeft) {
                p = internals_1.findSuccessor(tree, p);
                internals_1.swapNodeContents(p.node, current);
                current = p.node;
            }
            debt = current._red ? 0 : 1;
            current = internals_1.editRightChild(tree, current);
        }
        else if (hasLeft) {
            debt = current._red ? 0 : 1;
            current = internals_1.editLeftChild(tree, current);
        }
    }
    else {
        debt = current._red ? 0 : 1;
        current = internals_1.NONE;
    }
    if (p.parent.isNone()) {
        tree._root = current;
    }
    p.replace(current);
    if (current._red && p.parent.node._red) {
        current._red = false;
        debt = 0;
    }
    if (debt > 0) {
        var parent = (p = p.release()).node, gp = p.parent, sibling = internals_1.NONE, left = internals_1.NONE, right = internals_1.NONE;
        do {
            var branch = p.next;
            if (p.isNone()) {
                p.node._red = false;
                internals_1.updateCount(p.node);
                debt--;
            }
            else if (branch === 1 /* LEFT */) {
                if ((sibling = internals_1.editRightChild(tree, parent))._red) {
                    internals_1.swapNodeColors(parent, sibling);
                    internals_1.rotateLeft(gp, parent, sibling, tree);
                    p.node = sibling;
                    gp = p;
                    p = internals_1.PathNode.next(parent, p, 1 /* LEFT */);
                }
                else if (!(left = sibling._left)._red && !sibling._right._red) {
                    sibling._red = true;
                    if (parent._red) {
                        parent._red = false;
                        debt--;
                        if (p.isNone()) {
                            tree._root = current;
                        }
                    }
                    else {
                        current = parent;
                        p = p.release();
                        gp = p.parent;
                        parent = p.node;
                    }
                    internals_1.updateCount(parent);
                }
                else {
                    if (!sibling._right._red && left._red) {
                        left = internals_1.editLeftChild(tree, sibling);
                        p.next = 2 /* RIGHT */;
                        internals_1.swapNodeColors(sibling, left);
                        internals_1.rotateRight(p, sibling, left, tree);
                        sibling = left;
                    }
                    right = internals_1.editRightChild(tree, sibling);
                    internals_1.rotateLeft(gp, parent, sibling, tree);
                    sibling._red = parent._red;
                    parent._red = false;
                    right._red = false;
                    debt--;
                }
            }
            else {
                if ((sibling = internals_1.editLeftChild(tree, parent))._red) {
                    internals_1.rotateRight(gp, parent, sibling, tree);
                    internals_1.swapNodeColors(parent, sibling);
                    p.node = sibling;
                    gp = p;
                    p = internals_1.PathNode.next(parent, p, 2 /* RIGHT */);
                }
                else if (!(right = sibling._right)._red && !sibling._left._red) {
                    sibling._red = true;
                    if (parent._red) {
                        parent._red = false;
                        debt--;
                        if (p.isNone()) {
                            tree._root = current;
                        }
                    }
                    else {
                        current = parent;
                        p = p.release();
                        gp = p.parent;
                        parent = p.node;
                    }
                    internals_1.updateCount(parent);
                }
                else {
                    if (!sibling._left._red && right._red) {
                        right = internals_1.editRightChild(tree, sibling);
                        p.next = 1 /* LEFT */;
                        internals_1.swapNodeColors(sibling, right);
                        internals_1.rotateLeft(p, sibling, right, tree);
                        sibling = right;
                    }
                    left = internals_1.editLeftChild(tree, sibling);
                    internals_1.rotateRight(gp, parent, sibling, tree);
                    sibling._red = parent._red;
                    parent._red = false;
                    left._red = false;
                    debt--;
                }
            }
        } while (debt > 0);
    }
    if (p.isActive()) {
        current = internals_1.PathNode.releaseAndRecount(p, current);
    }
    tree._size--;
    return C.commit(tree);
}
exports.remove = remove;
//# sourceMappingURL=remove.js.map